#!/usr/bin/python3
#
#
"""
     This program does a scan of impedance versus frequency
     over a range specified by the call
     
     The call is:
     
     ./z_scan.py b u n filename plottype logtype
     
     where  b is the start frequency in Hz,
			u is the stop frequency, in Hz,
			n is th number of frequencies
			filename is the name to give to the png and textfile of the results
            plottype is either 'g', 'v' or 'r' for gamma, VSWR or return loss, respectively
            logtype is 1 for linear or 0 for logarithmic

			NOTE: '.png' and '.txt' will be added to the filename to create the files
			
	Jim Koehler, Comox, April, 2019
"""
import numpy as np
import math
import sys
import serial
import time
import os

def rect_to_polar(x, y):
	mag = math.sqrt(x**2 + y**2)
	ph = math.tan(y/x)
	return mag, ph
	
def polar_to_rect(mag, ph):
	x = mag * math.cos(ph)
	y = mag * math.sin(ph)
	return x, y

def get_cal():
	r = np.loadtxt("calibration.txt")
	return r
    
def get_freq_cal():
    v = np.loadtxt("frequency_calibration.txt")
    f_corr = v[1] / v[0]
#    print(f_corr)
    return f_corr
	
def calibrate( inp, f, c):
	rh = inp[0]  # inp is the uncorrected list with rho magnitude and phase
	ph = inp[1]
	i = round(f / 1e6) - 1 # points to the correct calibration number
	rh = rh * c[i, 0] # correct magnitude
	ph = ph - c[i, 1] # correct phase
	a, b = polar_to_rect(rh, ph) #convert to rectangular
	return a + b * 1j
	

def do_scan(b, u, n, logtype):
    R = complex(50.0, 0)
    TWO = complex(2.0, 0)
    r = [0, 0]
    comm = serial.Serial('/dev/ttyACM0', baudrate = 115200, timeout = 0.3)
    comm.readlines()    # flush the read buffer
    comm.write(s2b("b " + b + "\r")) 
    time.sleep(0.2)
    comm.write(s2b("u " + u + "\r")) 
    time.sleep(0.2)
    comm.write(s2b("n " + n + "\r")) 
    time.sleep(0.2)    
    if (float(logtype) < 0.5):
        comm.write(s2b("o\r"))
    else:
        comm.write(s2b("l\r"))        
    time.sleep(0.2)    
    comm.readlines()
    
   
    c = get_cal()
    
    frequency_correction_factor = get_freq_cal()
    
    comm.write(s2b("g\r"))
    
    fh=open("tmp.txt", 'wb')   
    for i in range(int(n)):
        v = comm.readline()
        fh.write(v)
    fh.close()
    
    fh=open("tmp.txt", 'rb')
    fh_out = open("gamma.txt", 'wb')
    fh_out.write(s2b("#    Gamma vs frequency\n"))
    fh_out.write(s2b("# \n"))
    fh_out.write(s2b("# freq         mag              phase              VSWR              ReturnLoss\n"))
    fh_out.write(s2b("# \n"))
   
   
    for i in range(int(n)):
        v = str.split(b2s(fh.readline()))
        r[0] = float(v[1]) / float(v[3])
        r[1] = float(v[2]) - float(v[4])
        rho = calibrate(r, float(v[0]), c)
        z = R * rho/(TWO - rho)
        if (z.real < 0.0):
            z = complex(0.0, 0.0)
        gamma = (z - R) / (z + R)
        a, b = rect_to_polar(gamma.real, gamma.imag)
        f = round(int(v[0]) * frequency_correction_factor)
        a = abs(a)
        if (a < 1.0):
            vswr = (1 + a) / (1 - a)
            ret_loss = 20.0 * math.log10 (a)
        else:
            vswr = 100.0
            ret_loss = 0.0
        if (vswr > 100.0):
            vswr = 100.0
        fh_out.write(s2b(str(f) + ' ' + str(a) + ' ' + str(b) + ' ' + str(vswr) + ' ' + str(ret_loss) + '\n'))
 
    fh.close()
    fh_out.close()
	

def b2s(message):
    '''Byte to string'''
    return bytes.decode(message)

def s2b(message):
    '''string to bytes'''
    return bytearray(message, "ascii")

def main():
    b = sys.argv[1] # the first argument of the command line
    u = sys.argv[2] # the second argument of the command line
    n = sys.argv[3] # the third argument of the command line
    filename = sys.argv[4]
    logtype = sys.argv[5]
    do_scan(b, u, n, logtype)
    string = "-c VSWRplot.gp " + str(b) + " " + str(u) + " " + str(logtype)
    os.system("start wgnuplot " + string)
    os.system("feh gamma.png")
    os.system("mv gamma.png " + filename + ".png")
    os.system("mv gamma.txt " + filename + ".txt")   

if __name__=="__main__":
    main()
    
